library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;

entity nexys4_xadc_minimal is port(
	clk			: in std_logic;
	xadc_p		: in std_logic;
	xadc_n		: in std_logic;
	xadc_vauxp3 : in std_logic;
	xadc_vauxn3	: in std_logic;
	RsTx		: out std_logic);
end nexys4_xadc_minimal;

architecture Behavioral of nexys4_xadc_minimal is

component xadc_wiz_0 is port(
    daddr_in        : in  STD_LOGIC_VECTOR (6 downto 0);     -- Address bus for the dynamic reconfiguration port
    den_in          : in  STD_LOGIC;                         -- Enable Signal for the dynamic reconfiguration port
    di_in           : in  STD_LOGIC_VECTOR (15 downto 0);    -- Input data bus for the dynamic reconfiguration port
    dwe_in          : in  STD_LOGIC;                         -- Write Enable for the dynamic reconfiguration port
    do_out          : out  STD_LOGIC_VECTOR (15 downto 0);   -- Output data bus for dynamic reconfiguration port
    drdy_out        : out  STD_LOGIC;                        -- Data ready signal for the dynamic reconfiguration port
    dclk_in         : in  STD_LOGIC;                         -- Clock input for the dynamic reconfiguration port
    vauxp3          : in  STD_LOGIC;                         -- Auxiliary Channel 3
    vauxn3          : in  STD_LOGIC;
    busy_out        : out  STD_LOGIC;                        -- ADC Busy signal
    channel_out     : out  STD_LOGIC_VECTOR (4 downto 0);    -- Channel Selection Outputs
    eoc_out         : out  STD_LOGIC;                        -- End of Conversion Signal
    eos_out         : out  STD_LOGIC;                        -- End of Sequence Signal
    alarm_out       : out STD_LOGIC;                         -- OR'ed output of all the Alarms
    vp_in           : in  STD_LOGIC;                         -- Dedicated Analog Input Pair
    vn_in           : in  STD_LOGIC);
end component;

component uart is
	generic(
	clk_freq: integer;
	baudrate: integer);
	port(
	clk : in std_logic;
	rdy_out : out std_logic;
	start_in : in std_logic:='0';
	start_out : out std_logic:='0';
	tx_uart : out std_logic:='1';
	rx_uart : in std_logic:='0';
	byte_out : out std_logic_vector(7 downto 0);
	byte_in : in std_logic_vector(7 downto 0));
end component;

signal drdy_xadc: std_logic:='0';
signal eoc_xadc: std_logic:='0';
signal do_xadc: std_logic_vector(15 downto 0):=(others => '0'); 
 
begin

u0: uart
	GENERIC MAP(
	clk_freq => 100000000,
	baudrate => 115200)
	PORT MAP(
	clk => clk,
	rdy_out => open,
	start_in => drdy_xadc,
	start_out => open,
	tx_uart => RsTx,
	rx_uart => '0',
	byte_out => open,
	byte_in => do_xadc(15 downto 8));

xadc : xadc_wiz_0 port map(
    daddr_in	=> "0010011", -- 19,
    den_in		=> eoc_xadc,
    di_in		=> (others => '0'),
    dwe_in		=> '0',
    do_out		=> do_xadc, -- Daten
    drdy_out	=> drdy_xadc, -- wird verwendet um Daten zu erfassen
    dclk_in		=> clk,
    vauxp3		=> xadc_vauxp3,
    vauxn3		=> xadc_vauxn3,
    busy_out	=> open,
    channel_out	=> open, --channel_xadc
    eoc_out		=> eoc_xadc,
    eos_out		=> open,
    alarm_out	=> open,
    vp_in		=> xadc_p,
    vn_in		=> xadc_n);
	
end Behavioral;