library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity uart is 
	generic(
	clk_freq: integer;
	baudrate: integer);
	port(
	clk : in std_logic;
	rdy_out : out std_logic;
	start_in : in std_logic;
	start_out : out std_logic:='0';
	tx_uart : out std_logic:='1';
	rx_uart : in std_logic;
	byte_out : out std_logic_vector(7 downto 0):=(others => '0');
	byte_in : in std_logic_vector(7 downto 0));
end uart;

architecture Verhalten of uart is

constant clockdivider_max: integer:=clk_freq/baudrate;
signal tx_clockdivider: integer range 0 to clockdivider_max:=0;
signal rx_clockdivider: integer range 0 to clockdivider_max:=0;
signal rx_state: integer range 0 to 2:=0;
signal tx_state: integer range 0 to 2:=0;
signal tx_bits: std_logic_vector(8 downto 0):=(others => '0');
signal rx_bits: std_logic_vector(8 downto 0):=(others => '0');
signal rx_bitcounter: integer range 0 to 9:=0;
signal tx_bitcounter: integer range 0 to 9:=0;
signal rx_old: std_logic:='0';

begin

process begin
wait until rising_edge(clk);
	rx_old <= rx_uart;
	rx_clockdivider <= rx_clockdivider +1;
	if rx_clockdivider = clockdivider_max then
		rx_clockdivider <= 0;
	end if;
	
	if    rx_state = 0 then --warten auf start
		if rx_old = '1' and rx_uart = '0' then
			rx_state <= 1;
			rx_clockdivider <= 0;
			rx_bitcounter <= 0;
		end if;
	elsif rx_state = 1 then --empfangen
		if rx_clockdivider = clockdivider_max/2 then
			rx_bits <= rx_uart & rx_bits(8 downto 1);
			rx_bitcounter <= rx_bitcounter +1;
			if rx_bitcounter = 9 then
				if rx_bits(0) = '0' and rx_uart = '1' then
					start_out <= '1';
					byte_out <= rx_bits(8 downto 1);
				end if;
				rx_state <= 2;
			end if;	
		end if;
	elsif rx_state = 2 then --ende
		start_out <= '0';
		rx_state <= 0;
	end if;
end process;

process begin
wait until rising_edge(clk);
	tx_clockdivider <= tx_clockdivider +1;
	if tx_clockdivider = clockdivider_max then
		tx_clockdivider <= 0;
	end if;
	
	if    tx_state = 0 then
		rdy_out <= '1';
		if start_in = '1' then
			rdy_out <= '0';
			tx_bits <= "1" & byte_in;
			tx_uart <= '0';
			tx_state <= 1;
			tx_clockdivider <= 0;
			tx_bitcounter <= 0;
		end if;
	elsif tx_state = 1 then
		if tx_clockdivider = clockdivider_max then
			tx_uart <= tx_bits(tx_bitcounter);
			tx_bitcounter <= tx_bitcounter +1;
			if tx_bitcounter = 8 then
			 tx_state <= 2;
			end if;	
		end if;
	elsif tx_state = 2 then
		if tx_clockdivider = clockdivider_max then
			tx_uart <= '1';
			tx_state <= 0;
			rdy_out <= '1';
		end if;
	end if;
end process;

end Verhalten;